# addusers.pl script used to create bulk users
use Win32::UserAdmin;
use Win32::NetResource;
use Win32::FileSecurity;

$verbose = false;
$arg = shift @ARGV;
if($arg eq '/v') {
    $verbose = true;
    $arg = shift @ARGV;
}
$domain = $arg;
$userfile = shift @ARGV;


# srand is used to seed the randomization routine
# for creating user passwords. The series of 'for'
# loops then propogates an array with possible
# password characters.
srand( time() ^ ($$ + ($$ << 15)) );
$char_index = 0;
for($num = 50; $num < 58; $num++){$passchar[$char_index++] = chr($num);}
for($num = 65; $num < 73; $num++){$passchar[$char_index++] = chr($num);}
for($num = 74; $num < 79; $num++){$passchar[$char_index++] = chr($num);}
for($num = 80; $num < 91; $num++){$passchar[$char_index++] = chr($num);}
for($num = 97; $num < 108; $num++){$passchar[$char_index++] = chr($num);}
for($num = 109; $num < 111; $num++){$passchar[$char_index++] = chr($num);}
for($num = 112; $num < 123; $num++){$passchar[$char_index++] = chr($num);}


# first we get the domain controller for the domain
# we are interested in
Win32::UserAdmin::GetDomainController('', $domain, $server);

# need to get the list of current users
# this will also inlcude the names of the
# computers in the domain, but we can ignore these
Win32::UserAdmin::GroupGetMembers($server,'Domain Users',\@users)
    || die "Unable to list current users";

# place the current usernames in a hash instead
# of an array for easier lookup.
print "Current Users:\n" if $verbose;
foreach $user(@users) {
    $current_users{$user} = " ";
    print "$user\n" if $verbose;
}
print "\n" if $verbose;

open(NEWUSERS, $userfile) or die "cannot open user file $!\n";
open LOGFILE, '>>.\out.log' or die "cannot open log file $!\n";
TEMPLATE: while($line = <NEWUSERS>) {
    ($template, @template_config) = split(/\s*[,;\n]\s*/,$line);
    chomp($template);
    if($current_users{$template}) {

        Win32::UserAdmin::UserGetAttributes($server, $template, 3, \%temp_info)
                || die "Unable to get template information\n";

        $_ = $temp_info{'USER_USR_COMMENT'};
        (/User's comment\s+Index:\s+(\d+)/);
        $template_index = $1;
    } else {
        die "Template User $template does not Exist";
    }

    while($line = <NEWUSERS>) {
        foreach $key(keys %temp_info) {
            $new_user{$key} = $temp_info{$key};
        }
        next TEMPLATE if $line eq "\n";
        chop($line);
        ($fullname, $comment) = split(/\s*[,;]\s*/,$line);

        $new_user{'USER_FULL_NAME'} = $fullname;
        $new_user{'USER_COMMENT'} = $comment;

        @names = split(/\s+/,$fullname);
        $username = ""; $initials = "";
        foreach (@names) {
            /(\w)./;
            $initials .= "\L$1";
        }
        $tempname = $initials . $template_index;
        $index = 1;
        while($current_users{($username = $tempname . $index++)}){}
        $new_user{'USER_NAME'} = $username;

        $password = "";
        for($num = 0; $num < 8; $num++) {
            $password .= $passchar[rand($char_index)];
        }
        $new_user{'USER_PASSWORD'} = $password;

        $_ = $temp_info{'USER_PROFILE'};
        s/(.*)$template(.*)/$1$username$2/;
        $new_user{'USER_PROFILE'} = $_;
        $_ = $temp_info{'USER_SCRIPT_PATH'};
        s/(.*)$template(.*)/$1$username$2/;
        $new_user{'USER_SCRIPT_PATH'} = $_;

        # we need to make the home directory and share it


        # first, get server and share name from template user
        $_ = $temp_info{'USER_HOME_DIR'};
        /(\\\\.*)\\(.*)/;
        $home_server = $1;
        $template_share = $2;
        Win32::UserAdmin::NetShareGetInfo($home_server, $template_share, \%share_info);

        foreach $key(keys %share_info) {
            print $share_info{$key},"\n";
        }

        $_ = $share_info{'path'};
        s/(.*)$template(.*)/$1$username$2/;
        $homedir_absolute_path = $_;

        $_ = $temp_info{'USER_HOME_DIR'};
        s/(.*)$template(.*)/$1$username$2/;
        $new_user{'USER_HOME_DIR'} = $_;


        print "home dir path: $homedir_absolute_path\n";
        `mkdir $homedir_absolute_path`;

        $share_info{'SHARE_NET_NAME'} = "$new_user{'USER_NAME'}\$";
        $share_info{'SHARE_REMARK'} = "$new_user{'USER_NAME'} home directory";
        $share_info{'SHARE_PATH'} = $homedir_absolute_path;
        $share_info{'SHARE_PASSWORD'} = "";

        $new_user{'USER_HOME_DIR'} = $home_server."\\".$share_info{'netname'};

        Win32::UserAdmin::NetShareAdd($home_server, \%share_info);

        foreach $key(keys %temp_info) {
            print "$key:\t\t$temp_info{$key}\t\t$new_user{$key}\n" if $verbose;
        }

        if(Win32::UserAdmin::UserCreate($server, 3, \%new_user) || die "user creation failed") {

            $dir_sec = Win32::FileSecurity::MakeMask(qw(FULL GENERIC_ALL));
            $hash{Administrator} = $dir_sec;
            $hash{$username} = $dir_sec;
            Win32::FileSecurity::Set($new_user{'USER_HOME_DIR'}, \%hash);

            Win32::UserAdmin::UserGetGroups($server, $new_user{"USER_NAME"}, \@groups);
            foreach $group(@groups) {
                Win32::UserAdmin::GroupAddUsers($server, $group, $new_user{"USER_NAME"});
            }

            $current_users{$new_user{"USER_NAME"}} = " ";
            print LOGFILE $new_user{"USER_FULL_NAME"}, ", ", $new_user{"USER_NAME"}, ", ", $new_user{"USER_PASSWORD"}, "\n";
        } else {
            print LOGFILE "Unable to create user ", $new_user{"USER_NAME"}, " for ", $new_user{"USER_FULL_NAME"}, "\n";
        }
    }
}

close NEWUSERS;
close LOGFILE;
